Material resource
{
	MaterialType _type = Blended;
	VideoProgram _passes
	[ 
		"normal"
	]
}

VideoProgram normal
{
	VertexLayout _vertexLayout = "MeshLayout\TerrainDecalLayout.rsc";
	VertexProgram _vertexProgram = "vs";
	PixelProgram _pixelProgram = "ps";
	RenderState _renderState = "Material\TerrainDecal\TerrainDecalMaterial.rsc:rs";
}

VertexProgram vs
{
	VertexLayout _input = "MeshLayout/TerrainDecalLayout.rsc";
	String _includes
	[
		"Material/Instancing.hlsl"
		"Material/Lighting.hlsl"
		"Material/UnderWater.hlsl"
	]

	Output _outputs 
	[
		{ String _type = "float4"; String _name = "position";		Usage _usage = Position;			int _index = 0; }
		{ String _type = "float4"; String _name = "texcoord";		Usage _usage = TextureCoordinate;	int _index = 0; }
		{ String _type = "float4"; String _name = "lightFogAlpha";	Usage _usage = TextureCoordinate;	int _index = 1; }
		{ String _type = "float";  String _name = "underWater";		Usage _usage = TextureCoordinate;	int _index = 2; }
	]

	String _functions
	[
		"
			void Common(float4x4 localToWorld, float4 position, float3 normal, float4 texcoord, out float3 worldPosition, out float4 outputPosition,
				out float4 outputTexcoord, out float4 lightFogAlpha, out float underWater)
			{
				position.z /= 256.0;
				worldPosition = mul((float3x4)localToWorld, float4(position.xyz, 1.0));
				outputPosition = mul(gc_worldToProjection, float4(worldPosition, 1.0));
	
				normal = normalize(normal * 2 - 1);
	
				lightFogAlpha.xyz = GetLightingFog(normal, outputPosition.z);
				lightFogAlpha.w = position.w / 255.0f;
	
				outputTexcoord.xy = texcoord.xy / 16.0;
				outputTexcoord.zw = texcoord.zw / 4096.0;
				underWater.x = GetUnderWaterAmount(position.z);
			}
		"
	]

	Variant _variants
	[
		{
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(0);
				float3 worldPosition;
				Common(localToWorld, input.position, input.normal, input.texcoord, worldPosition, output.position, output.texcoord, output.lightFogAlpha, output.underWater);
			";
		}
		{
			Output _outputs 
			[
				{ String _type = "float4"; String _name = "shadowProjection";	Usage _usage = TextureCoordinate;	int _index = 3; }
			]
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(0);
				float3 worldPosition;
				Common(localToWorld, input.position, input.normal, input.texcoord, worldPosition, output.position, output.texcoord, output.lightFogAlpha, output.underWater);
				output.shadowProjection = mul(lc_shadowProjection0, float4(worldPosition, 1.0f));
			";
		}
		{
			Output _outputs 
			[
				{ String _type = "float4"; String _name = "shadowProjection";	Usage _usage = TextureCoordinate;	int _index = 3; }
			]
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(0);
				float3 worldPosition;
				Common(localToWorld, input.position, input.normal, input.texcoord, worldPosition, output.position, output.texcoord, output.lightFogAlpha, output.underWater);
				output.shadowProjection = mul(lc_shadowProjection0, float4(worldPosition, 1.0f));
			";
		}
		{
			Output _outputs 
			[
				{ String _type = "float4"; String _name = "shadowProjection";	Usage _usage = TextureCoordinate;	int _index = 3; }
			]
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(0);
				float3 worldPosition;
				Common(localToWorld, input.position, input.normal, input.texcoord, worldPosition, output.position, output.texcoord, output.lightFogAlpha, output.underWater);
				output.shadowProjection = mul(lc_shadowProjection0, float4(worldPosition, 1.0f));
			";
		}
	]
}

PixelProgram ps
{
	VertexProgram _input = "vs";
	String _includes
	[
		"Material/Texture.hlsl"
		"Material/Shadows.hlsl"
	]

	Texture _textureList
	[
		{ bool _2d = true;	String _name = "diffuse";	int _bindIndex = 0;	bool _exposed = true; }
		{ bool _2d = true;	String _name = "tiles";		int _bindIndex = 1;	bool _exposed = true; }
		{ bool _2d = true;	String _name = "snow";		int _bindIndex = 2;	bool _exposed = true; }
		{ bool _2d = true;	String _name = "shadowMap";	int _bindIndex = 5;	bool _exposed = false; bool _shadowMap = true; }
	]

	String _functions
	[
		"
		float4 GetColor(float4 texcoord, float4 lightFogAlpha, float underWater, float shadow)
		{
			float4 diff = Sample2D(diffuse, diffusesampler, texcoord.xy);
			float4 sno = Sample2D(snow, snowsampler, texcoord.xy);
			diff.xyz = lerp(diff.xyz, sno.xyz, diff.w * pc_userPosition.x);
			float4 mask = Sample2D(tiles, tilessampler, texcoord.zw);
    
			float4 output;
    
			output.xyz = ComputeLighting(lightFogAlpha.xyz, diff.xyz, shadow);
			output.xyz = lerp(output.xyz, float3(0, 0, 0), saturate(underWater.x));
			output.w = mask.y * lightFogAlpha.w;
	
			return output;
		}

		"
	]

	Variant _variants
	[
		{
			String _text = "output.color = GetColor(input.texcoord, input.lightFogAlpha, input.underWater, 1);";
		}
		{
			String _text = "output.color = GetColor(input.texcoord, input.lightFogAlpha, input.underWater, GetShadowLow(input.shadowProjection));";
		}
		{
			String _text = "output.color = GetColor(input.texcoord, input.lightFogAlpha, input.underWater, GetShadowMedium(input.shadowProjection, pc_texelSize.x));";
		}
		{
			String _text = "output.color = GetColor(input.texcoord, input.lightFogAlpha, input.underWater, GetShadowHigh(input.shadowProjection, pc_texelSize.x));";
		}
	]
}
